@@ -1,3 +1,4 @@
+#include <QMessageBox>
 #include "importevents.h"
 #include "ui_importevents.h"
 #include <QFileDialog>
@@ -7,6 +8,8 @@
 #include <QSqlQuery>
 #include <QSqlError>
 #include <QSqlRecord>
+#include <QtNetwork>
+#include <QtEndian>
 
 //QSqlRecord rec;
 //QSqlDatabase db;
@@ -14,11 +17,22 @@
 
 int iEventID = -1;
 
+const int cFileVersion     = 1;
+const int cHeaderSize      = 89;
+const int cTeamSize        = 42;
+const int cMatchSize       = 9;
+const int cLeagueNameChars = 16;
+const int cTeamNameChars   = 10;
+
+QHash<int, QString> hTeamIDs;
+QHash<int, int> hiTeamIDs;
+
 importevents::importevents(QWidget *parent) :
     QDialog(parent),
     ui(new Ui::importevents)
 {
     ui->setupUi(this);
+
 }
 
 importevents::~importevents()
@@ -56,20 +70,348 @@ void importevents::on_pbTeamRooster_clicked()
     importTeamRooster();
 }
 
+void importevents::on_pbTeamStats_clicked()
+{
+    importTeamStats();
+}
+
+void importevents::importTeamStats()
+{
+//    LoadHeader();
+}
+
+void importevents::LoadMatch()
+{
+}
+
+void importevents::LoadTeam()
+{
+}
+
+void importevents::LoadHeader(int iSoccerDbID, int iEventRowID, QString sLatestGameDate)
+{
+    QString sSelectedSoccerDBID;
+    QString sCurrentGameDate;
+//    QString sLatestGameDate;
+    QString sDay;
+    QString sMonth;
+    QNetworkAccessManager manager;
+    QEventLoop loop;
+    QString sURL;
+    QString path;
+    QSqlQuery qry;
+
+//    sLatestGameDate = getLatestGameDate(iEventID);
+//    qDebug() << sLatestGameDate;
+
+    sSelectedSoccerDBID.setNum(iSoccerDbID);
+    iEventID = iEventRowID;
+    sURL = "http://www.aragon.ws/soccerdb/util/ppccreate.php?id=" + sSelectedSoccerDBID;
+    qDebug() << sURL;
+
+    // TO REMOVE COMMENT WHEN ENABLING ONLINE DOWNLOAD AGAIN
+    QNetworkReply *reply = manager.get( QNetworkRequest(QUrl(sURL)));
+    QObject::connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
+    loop.exec();
+
+
+#ifdef Q_OS_LINUX
+        // NOTE: We have to store database file into user home folder in Linux
+        path = QDir::home().path();
+        path.append(QDir::separator()).append(".qsportsevent").append(QDir::separator()).append("tempfile.ts");
+        path = QDir::toNativeSeparators(path);
+#else
+        path = "tempfile.ts" ;
+        // NOTE: File exists in the application private folder, in Symbian Qt implementation
+#endif
+    qDebug() << "path=" <<path;
+    QFile file( path );
+
+    // TO REMOVE COMMENT WHEN ENABLING ONLINE DOWNLOAD AGAIN
+    file.open(QIODevice::WriteOnly);
+    file.write(reply->readAll());
+    file.close();
+    delete reply;
+
+
+    file.open(QIODevice::ReadOnly);
+    QDataStream in(&file);    // read the data serialized from the file
+    QString str;
+
+    char pline[cLeagueNameChars * 4] = {0};
+    int readLength ;
+    int iGroupID;
+
+    qint8 Version;
+    qint16 Year;
+    qint8 TeamCount;
+    qint8 RoundCount;
+    qint16 MatchCount;
+    qint16 PlayedMatchCount;
+
+    qint8 WinPoints;
+    qint8 DrawPoints;
+    qint8 LossPoints;
+
+    qint8 TableSort;
+    qint64 TableLines;
+
+
+    readLength = in.readRawData(pline, 4);
+    qDebug() << "readLength" << readLength ;
+    qDebug() << "File Type (should be mbTS)" << pline;
+
+    if (QString("mbTS").compare(QString(pline)) == 0)
+    {
+
+        deleteGames(iEventID);
+
+        qry.prepare("BEGIN");
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+        }
+
+
+        in >> Version;
+        qDebug() << Version;
+
+        char pline3[cLeagueNameChars * 4] = {0};
+        readLength = in.readRawData(pline3, cLeagueNameChars * 4);
+        qDebug() << "readLength" << readLength ;
+        qDebug() << "LeagueName" << pline3;
+
+        in >> Year;
+        Year =  qFromBigEndian(Year);
+        qDebug() << Year;
+        in >> TeamCount;
+  //      qDebug() << TeamCount;
+        in >> RoundCount;
+  //      qDebug() << RoundCount;
+        in >> MatchCount;
+        MatchCount =  qFromBigEndian(MatchCount);
+//        qDebug() << MatchCount;
+        in >> PlayedMatchCount;
+        PlayedMatchCount =  qFromBigEndian(PlayedMatchCount);
+//        qDebug() << PlayedMatchCount;
+
+        in >> WinPoints;
+//        qDebug() << WinPoints;
+        in >> DrawPoints;
+//        qDebug() << DrawPoints;
+        in >> LossPoints;
+//        qDebug() << LossPoints;
+        in >> TableSort;
+//        qDebug() << TableSort;
+        in >> TableLines;
+//        qDebug() << TableLines;
+
+        qint8 BonusPoints;
+        qint8 HiddenPoints;
+        int teamID;
+        hTeamIDs.clear();
+        QString sTeamName;
+//      Do away with the group table. Use event id instead.
+//        iGroupID = createGroup(QString(pline3));
+        iGroupID = iEventID ;
+
+        for (int i=0; i<TeamCount; i++)
+        {
+            readLength = in.readRawData(pline, cTeamNameChars * 4);
+//            qDebug() << "readLength" << readLength ;
+            qDebug() << "pline" << pline;
+            sTeamName = QString::fromUtf8(pline);
+
+            in >> BonusPoints;
+//            qDebug() << BonusPoints;
+
+            in >> HiddenPoints;
+//            qDebug() << HiddenPoints;
+
+//            teamID = createTeam(QString(pline), iGroupID );
+            teamID = createTeam(sTeamName, iGroupID );
+ //           hTeamIDs[i] = QString(pline);
+            hiTeamIDs[i] = teamID;
+            // Create hash between row id in the database and id from the file.
+        }
+        qry.prepare("COMMIT");
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+        }
+
+        qint8 match_Round;//      = (int)reader.ReadByte();
+        qint16 year;//         = (int)reader.ReadInt16();
+        qint8 month;//        = (int)reader.ReadByte();
+        qint8 day;//          = (int)reader.ReadByte();
+        qint8 HomeTeamId; // = (int)reader.ReadByte();
+        qint8 AwayTeamId; // = (int)reader.ReadByte();
+        qint8 HomeGoals; //  = (int)reader.ReadSByte();
+        qint8 AwayGoals; //  = (int)reader.ReadSByte();
+
+
+        qry.prepare("BEGIN");
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+        }
+
+
+        while (!in.atEnd())
+        {
+            in >> match_Round;
+//            qDebug() << match_Round;
+
+            in >> year;
+            year =  qFromBigEndian(year);
+//            qDebug() << year;
+
+            in >> month;
+//            qDebug() << month;
+
+            in >> day;
+//            qDebug() << day;
+
+            if (month <10)
+            {
+                sMonth = "0" + QString::number(month);
+            }
+            else
+            {
+                sMonth = QString::number(month);
+            }
+//            qDebug() << "Month:" << sMonth;
+            if (day <10)
+            {
+                sDay = "0" + QString::number(day);
+            }
+            else
+            {
+                sDay = QString::number(day);
+            }
+//            qDebug() << "Day:" << sDay;
+
+            sCurrentGameDate = QString::number(year).append("-").append(sMonth).append("-").append(sDay);
+            qDebug() << sCurrentGameDate;
+
+            in >> HomeTeamId;
+//            qDebug() << HomeTeamId;
+//            qDebug() << hTeamIDs.value(HomeTeamId);
+
+            in >> AwayTeamId;
+//            qDebug() << AwayTeamId;
+//            qDebug() << hTeamIDs.value(AwayTeamId);
+
+            in >> HomeGoals;
+//            qDebug() << HomeGoals;
+
+            in >> AwayGoals;
+//            qDebug() << AwayGoals;
+
+            if (sCurrentGameDate > sLatestGameDate)
+            {
+            createGame(QString(pline3),
+                       match_Round,
+                       sCurrentGameDate,
+                       QString(hTeamIDs.value(HomeTeamId)),
+                       QString(hTeamIDs.value(AwayTeamId)),
+                       HomeGoals,
+                       AwayGoals,
+                       hiTeamIDs.value(HomeTeamId),
+                       hiTeamIDs.value(AwayTeamId));
+            }
+        }
+
+        qry.prepare("COMMIT");
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+        }
+
+    }
+    else
+    {
+        //Download failed
+        qDebug() << "Download failed";
+        QString message = QString("Download Failed!");
+//        QMessageBox::warning ( this, tr("About"), message);
+    }
+
+    file.close();
+
+    file.remove();
+
+    return;
+}
+
+void importevents::deleteGames(int iEventID)
+{
+    QSqlQuery qry;
+    qry.prepare("DELETE FROM games "
+                "WHERE grouprowid=" + QString::number(iEventID));
+
+    if (!qry.exec())
+    {
+        qDebug() << qry.lastQuery();
+        qDebug() << qry.lastError();
+    }
+
+    return;
+}
+
+QString importevents::getLatestGameDate(int iEventID)
+{
+
+    QString sLatestGameDate;
+    QSqlQuery qry;
+
+/*
+    qry.prepare("SELECT MAX(gametime) "
+                "FROM games "
+                "WHERE homegoals<>-1 AND awaygoals<>-1 "
+                "AND grouprowid=" + QString::number(iEventID));
+*/
+    qry.prepare("SELECT gametime "
+                "FROM games ");
+//                "WHERE homegoals<>-1 AND awaygoals<>-1 "
+//                "AND grouprowid=" + QString::number(iEventID) + " ORDER BY gametime DESC LIMIT 1");
+
+    qDebug() << iEventID;
+
+    if (!qry.exec())
+    {
+        qDebug() << qry.lastQuery();
+        qDebug() << qry.lastError();
+    }
+
+    qry.next();
+
+    if (qry.isValid())
+    {
+        sLatestGameDate = qry.value(0).toString();
+        qDebug() << sLatestGameDate;
+        if (sLatestGameDate == "")
+        {
+            sLatestGameDate = "1800-01-01";
+        }
+     }
+    else
+    {
+        sLatestGameDate = "1800-01-01";
+    }
+
+    qDebug() << sLatestGameDate;
+
+    return sLatestGameDate;
+}
+
+
 void importevents::on_pbSoccerDB_clicked()
 {
-//    QSqlDatabase db;
-
-    #ifdef Q_OS_LINUX
-//                QString xmlpath(QDir::home().path());
-//                xmlpath.append(QDir::separator()).append("hsk.xml");
-//                xmlpath = QDir::toNativeSeparators(xmlpath);
-//        qDebug() << "XML path:" << xmlpath;
-//                QFile file(xmlpath);
-    #else
-//                QFile file( "./hsk.xml" );
-    #endif
-//    QFile file(ui->plainFileName);
 
     ui->lblImporting->setText("Importing...");
 
@@ -90,18 +432,6 @@ void importevents::on_pbSoccerDB_clicked()
 
 void importevents::on_pushImport_clicked()
 {
-//    QSqlDatabase db;
-
-    #ifdef Q_OS_LINUX
-//                QString xmlpath(QDir::home().path());
-//                xmlpath.append(QDir::separator()).append("hsk.xml");
-//                xmlpath = QDir::toNativeSeparators(xmlpath);
-//        qDebug() << "XML path:" << xmlpath;
-//                QFile file(xmlpath);
-    #else
-//                QFile file( "./hsk.xml" );
-    #endif
-//    QFile file(ui->plainFileName);
 
     ui->lblImporting->setText("Importing...");
 
@@ -234,11 +564,6 @@ void importevents::importSoccerDBLeagues()
 //            qDebug() << list1[i];
         }
         createEvent(list1[1], QString("Football"), list1[2], list1[0], list1[3].toInt());
-//        void importevents::createEvent(QString sEventName,
-//                                       QString sSport,
-//                                       QString sSeason,
-//                                       QString sRegion,
-//                                       int iSoccerDbID)
 
     }
 
@@ -422,7 +747,17 @@ void importevents::importGames(QString sFileName)
         iGroupID = createGroup(list1[0]);
         createTeam(list1[3], iGroupID );
         createTeam(list1[4], iGroupID );
-        createGame(list1);
+//        createGame(list1);
+        if (list1.size()>5)
+        {
+            // Results have been provided. Pass them.
+            createGame(list1[0], list1[1].toInt(), list1[2], list1[3], list1[4], list1[5].toInt(), list1[6].toInt());
+        }
+        else
+        {
+            // Results have not been provided. Pass -1.
+            createGame(list1[0], list1[1].toInt(), list1[2], list1[3], list1[4], -1, -1);
+        }
     }
 
     qry.prepare("COMMIT");
@@ -437,11 +772,10 @@ void importevents::importGames(QString sFileName)
 
 //"(name TEXT, sport TEXT, season TEXT, region TEXT)"))
 
-void importevents::createEvent(QString sEventName,
-                               QString sSport,
-                               QString sSeason,
-                               QString sRegion,
-                               int iSoccerDbID)
+int importevents::getEventID(QString sEventName,
+                            QString sSport,
+                            QString sSeason,
+                            QString sRegion)
 {
     QSqlQuery qry;
 
@@ -461,11 +795,29 @@ void importevents::createEvent(QString sEventName,
 
     iEventID = -1;
 
-    while (qry.next()) {
+    if (qry.next()) {
         iEventID = qry.value(0).toInt();
-//        qDebug() << iEventID;
+        qDebug() << qry.lastQuery();
+        qDebug() << iEventID;
      }
 
+    return iEventID;
+}
+
+
+void importevents::createEvent(QString sEventName,
+                               QString sSport,
+                               QString sSeason,
+                               QString sRegion,
+                               int iSoccerDbID)
+{
+    QSqlQuery qry;
+
+    iEventID = getEventID(sEventName,
+                          sSport,
+                          sSeason,
+                          sRegion);
+
     // Only insert if the Group does not already exist.
     if (iEventID < 1)
     {
@@ -574,11 +926,12 @@ int importevents::createGroup(QString sGroupName)
     // Only insert if the Group does not already exist.
     if (iGroupRowID < 0)
     {
-        qry.prepare("INSERT INTO teamgroups (name, eventrowid) VALUES "
-                    "(:name, :eventrowid)");
+        qry.prepare("INSERT INTO teamgroups (rowid, name, eventrowid) VALUES "
+                    "(:rowid, :name, :eventrowid)");
 
         qry.bindValue(":name", sGroupName);
         qry.bindValue(":eventrowid", iEventID);
+        qry.bindValue(":rowid", iEventID);
 
         if (!qry.exec())
         {
@@ -600,10 +953,11 @@ int importevents::createGroup(QString sGroupName)
 
 
 
-void importevents::createTeam(QString sTeamName, int iGroupRowID)
+int importevents::createTeam(QString sTeamName, int iGroupRowID)
 {
     QSqlQuery qry;
     int iTeamCount;
+    int rowID;
 
 //    qDebug() << "sTeamName" << sTeamName;
 
@@ -641,81 +995,117 @@ void importevents::createTeam(QString sTeamName, int iGroupRowID)
             qDebug() << qry.lastError();
             qDebug() << qry.executedQuery();
             qDebug() << qry.boundValues();
+            rowID = -1;
         }
         else
         {
+            rowID = qry.lastInsertId().toInt();
 //            qDebug() << "Team:" << sTeamName << iGroupRowID;
         }
     }
-
+    return rowID;
 }
 
-void importevents::createGame(QStringList sGameList)
+void importevents::createGame(QString sEventName,
+                              int iRound,
+                              QString sGameTime,
+                              QString sHomeTeamName,
+                              QString sAwayTeamName,
+                              int iHomeGoals,
+                              int iAwayGoals,
+                              int iHomeTeamID,
+                              int iAwayTeamID)
 {
-
+/*
+    qDebug() << sEventName;
+    qDebug() << iRound;
+    qDebug() << sGameTime;
+    qDebug() << sHomeTeamName;
+    qDebug() << sAwayTeamName;
+    qDebug() << iHomeGoals;
+    qDebug() << iAwayGoals;
+*/
     int ihometeamrowid;
     int iawayteamrowid;
     int igrouprowid;
     QSqlQuery qry;
 
-    qry.prepare("SELECT rowid FROM teams WHERE name=:name");
-    qry.bindValue(":name", sGameList[3]);
-
-    if (!qry.exec())
+    if (iHomeTeamID == -1)
     {
-        qDebug() << qry.lastQuery();
-        qDebug() << qry.lastError();
-    }
-    qry.next();
-    ihometeamrowid = qry.value(0).toInt();
+        qry.prepare("SELECT rowid FROM teams WHERE name=:name");
+        qry.bindValue(":name", sHomeTeamName);
 
-    qry.prepare("SELECT rowid FROM teams WHERE name=:name");
-    qry.bindValue(":name", sGameList[4]);
-//    qDebug() << "awayteam:" << sGameList[4];
-    if (!qry.exec())
-    {
-        qDebug() << qry.lastQuery();
-        qDebug() << qry.lastError();
-    }
-    qry.next();
-    iawayteamrowid = qry.value(0).toInt();
-//    qDebug() << "iawayteamrowid" << iawayteamrowid ;
-
-    qry.prepare("SELECT rowid FROM teamgroups WHERE name=:name");
-    qry.bindValue(":name", sGameList[0]);
-
-    if (!qry.exec())
-    {
-        qDebug() << qry.lastQuery();
-        qDebug() << qry.lastError();
-    }
-    qry.next();
-    igrouprowid = qry.value(0).toInt();
-
-    qry.prepare("INSERT INTO games (grouprowid, hometeamrowid, awayteamrowid, gametime, homegoals, awaygoals, round) VALUES "
-                "(:grouprowid, :hometeamrowid, :awayteamrowid, :gametime, :homegoals, :awaygoals, :round)");
-
-    qry.bindValue(":grouprowid", igrouprowid);
-//    qDebug() << "grouprowid" << igrouprowid;
-    qry.bindValue(":round", sGameList[1]);
-//    qDebug() << "round" << sGameList[1];
-    qry.bindValue(":gametime", sGameList[2]);
-//    qDebug() << "gametime" << sGameList[2];
-    qry.bindValue(":hometeamrowid", ihometeamrowid);
-//    qDebug() << "hometeamrwoid" << ihometeamrowid;
-    qry.bindValue(":awayteamrowid", iawayteamrowid);
-//    qDebug() << "awayteamrowid" << iawayteamrowid;
-    if (sGameList.size()>5)
-    {
-        qry.bindValue(":homegoals", sGameList[5]);
-        qry.bindValue(":awaygoals", sGameList[6]);
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+        }
+        qry.next();
+        ihometeamrowid = qry.value(0).toInt();
     }
     else
     {
-        qry.bindValue(":homegoals", 0);
-        qry.bindValue(":awaygoals", 0);
+        ihometeamrowid = iHomeTeamID;
     }
 
+    if (iAwayTeamID== -1)
+    {
+        qry.prepare("SELECT rowid FROM teams WHERE name=:name");
+        qry.bindValue(":name", sAwayTeamName);
+
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+        }
+        qry.next();
+        iawayteamrowid = qry.value(0).toInt();
+    }
+    else
+    {
+        iawayteamrowid = iAwayTeamID;
+    }
+
+
+    /*
+    if (iEventID != -1)
+    {
+        qry.prepare("SELECT rowid FROM teamgroups WHERE name=:name");
+        qry.bindValue(":name", sEventName);
+
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+        }
+        qry.next();
+        igrouprowid = qry.value(0).toInt();
+    }
+    else
+    {
+        igrouprowid = iEventID;
+    }
+    */
+
+    /*
+    // Update if the game already exists in the database.
+    qry.prepare("UPDATE games "
+                "SET gametime=:gametime, "
+                "homegoals=:homegoals, "
+                "awaygoals=:awaygoals "
+                "WHERE grouprowid=:grouprowid "
+                "AND hometeamrowid=:hometeamrowid "
+                "AND awayteamrowid=:awayteamrowid "
+                "AND round=:round");
+
+    qry.bindValue(":grouprowid", igrouprowid);
+    qry.bindValue(":round", iRound);
+    qry.bindValue(":gametime", sGameTime);
+    qry.bindValue(":hometeamrowid", ihometeamrowid);
+    qry.bindValue(":awayteamrowid", iawayteamrowid);
+    qry.bindValue(":homegoals", iHomeGoals);
+    qry.bindValue(":awaygoals", iAwayGoals);
+
     if (!qry.exec())
     {
         qDebug() << qry.lastQuery();
@@ -723,6 +1113,67 @@ void importevents::createGame(QStringList sGameList)
         qDebug() << qry.executedQuery();
         qDebug() << qry.boundValues();
     }
+    qDebug() << qry.numRowsAffected();
+    */
+    //If the game did not exist, INSERT it.
+//    if (qry.numRowsAffected() == 99)
+//    {
+        qry.prepare("INSERT INTO games (grouprowid, hometeamrowid, awayteamrowid, gametime, homegoals, awaygoals, round) VALUES "
+                    "(:grouprowid, :hometeamrowid, :awayteamrowid, :gametime, :homegoals, :awaygoals, :round)");
+
+//        qry.bindValue(":grouprowid", igrouprowid);
+        qry.bindValue(":grouprowid", iEventID);
+        qry.bindValue(":round", iRound);
+        qry.bindValue(":gametime", sGameTime);
+        qry.bindValue(":hometeamrowid", ihometeamrowid);
+        qry.bindValue(":awayteamrowid", iawayteamrowid);
+
+    //    if (iHomeGoals>-1)
+    //    {
+            qry.bindValue(":homegoals", iHomeGoals);
+            qry.bindValue(":awaygoals", iAwayGoals);
+    //    }
+    //    else
+    //    {
+    //        qry.bindValue(":homegoals", 0);
+    //        qry.bindValue(":awaygoals", 0);
+    //    }
+
+        if (!qry.exec())
+        {
+            qDebug() << qry.lastQuery();
+            qDebug() << qry.lastError();
+            qDebug() << qry.executedQuery();
+            qDebug() << qry.boundValues();
+        }
+//    }
+}
+
+void importevents::upgradeDatabase()
+{
+    QSqlQuery qry;
+    QString sVersion;
+
+    qry.prepare("SELECT value "
+                "FROM settings "
+                "WHERE name='version'");
+
+    if (!qry.exec())
+    {
+        qDebug() << qry.lastQuery();
+        qDebug() << qry.lastError();
+    }
+
+    qry.next();
+
+    if (qry.isValid())
+    {
+        sVersion = qry.value(0).toString();
+     }
+    else
+    {
+        sVersion = "0.0.7";
+    }
 
 }
 
@@ -735,7 +1186,8 @@ void importevents::createDatabase()
     #ifdef Q_OS_LINUX
                 // NOTE: We have to store database file into user home folder in Linux
                 QString path(QDir::home().path());
-                path.append(QDir::separator()).append("qsportsevent.db");
+                path.append(QDir::separator()).append(".qsportsevent").append(QDir::separator()).append("qsportsevent.db");
+//                path.append(QDir::separator()).append("qsportsevent.db");
                 path = QDir::toNativeSeparators(path);
 //        qDebug() << "Db path:" << path;
                 db.setDatabaseName(path);
@@ -807,5 +1259,6 @@ void importevents::createDatabase()
         qDebug() << qry.lastQuery();
         qDebug() << qry.lastError();
     }
+
 }
 
